package io.skore.smooch_plugin

import android.app.Activity
import android.content.Intent
import android.util.Log
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import io.smooch.core.*
import io.smooch.ui.ConversationActivity
import java.util.*
import kotlin.collections.HashMap

class SmoochPlugin(private val activity: Activity, private val channel: MethodChannel) : MethodCallHandler {


    companion object {
        @JvmStatic
        fun registerWith(registrar: Registrar) {
            val channel = MethodChannel(registrar.messenger(), "smooch_plugin")
            channel.setMethodCallHandler(SmoochPlugin(registrar.activity(), channel))
        }
        fun backButton(){

        }

    }



    override fun onMethodCall(call: MethodCall, result: Result) {
        when (call.method) {
            "initialize" -> initialize(call.arguments as? HashMap<String, Any>, result)
            "show" -> show(call.arguments as? HashMap<String, Any>)
            "logout" -> logout()
            else -> result.notImplemented()
        }
    }

    private fun initialize(arguments: HashMap<String, Any>?, result: Result) {

        val arguments = arguments?.let { it } ?: return result.success(-1)
        val appId = (arguments["appId"] as? String)?.let { it } ?: return result.success(-1)
        val userId = (arguments["userId"] as? String)?.let { it } ?: return result.success(-1)
        val smoochToken = (arguments["smoochToken"] as? String)?.let { it }
                ?: return result.success(-1)
        Smooch.init(activity.application, Settings(appId)) { initResponse ->
            if (initResponse.status == 200) {
                Smooch.login(userId, smoochToken) { loginResponse ->
                    if (loginResponse.status == 200) {
                        addDelegate()
                        result.success(0)
                    } else {
                        result.success(-1)
                    }
                }
            } else {
                result.success(-1)
            }
        }
        addDelegate()

    }



    private fun addDelegate() {
        Smooch.getConversation()?.let {
            it.delegate = object : Conversation.Delegate {
                override fun onInitializationStatusChanged(p0: InitializationStatus?) {

                }

                override fun onSmoochHidden() {
                }

                override fun onPaymentProcessed(p0: MessageAction?, p1: PaymentStatus?) {
                }

                override fun onMessageSent(p0: Message?, p1: MessageUploadStatus?) {
                }

                override fun onLoginComplete(p0: LoginResult?) {
                }

                override fun onSmoochShown() {
                }

                override fun onCardSummaryLoaded(p0: CardSummary?) {
                }

                override fun onSmoochConnectionStatusChanged(p0: SmoochConnectionStatus?) {
                }

                override fun onMessagesReceived(p0: Conversation?, p1: MutableList<Message>?) {
                }

                override fun onConversationEventReceived(p0: ConversationEvent?) {
                }

                override fun onMessagesReset(p0: Conversation?, p1: MutableList<Message>?) {
                }

                override fun shouldTriggerAction(messageAction: MessageAction?): Boolean {
                    messageAction?.let {
                        val uri = it.uri
                        if (it.type != "webview" || uri.contains("viaapp_terms")) {
                            return true
                        } else {
                            channel.invokeMethod("handleMessageAction", uri)

                        }
                        return true
                    }
                    return true
                }

                override fun onLogoutComplete(p0: LogoutResult?) {
                    channel.invokeMethod("", true)
                }

                override fun onUnreadCountChanged(p0: Conversation?, p1: Int) {
                    channel.invokeMethod("updateChatUnreadCount", p1)
                }
            }
        }
    }

    private fun show(arguments: HashMap<String, Any>?) {

        channel.invokeMethod("shouldSendStart", null, object : MethodChannel.Result {
            override fun notImplemented() {

            }

            override fun error(p0: String?, p1: String?, p2: Any?) {

            }

            override fun success(result: Any?) {
                (result as? Boolean)?.let {
                    if (it) {
                        val message = Message("iniciar")
                        Smooch.getConversation()?.sendMessage(message)
                    }
                }
            }

        })

        arguments?.let {
            if (it.containsKey("properties")) {
                (it["properties"] as? HashMap<String, Any>)?.let {
                    User.getCurrentUser().addProperties(it)
                }
            }
        }


        val intent = Intent(activity, SmoochActivity::class.java)
        activity.startActivity(intent)
    }

    private fun logout() {
        Smooch.logout(null)
    }
}
