//
// Created by João Paulo Ros on 06/12/17.
// Copyright (c) 2017 PremierSoft Solucoes Mobile. All rights reserved.
//

import Foundation
import Smooch
import Flutter

class NavigationController: UINavigationController {
    
    override open var supportedInterfaceOrientations: UIInterfaceOrientationMask {
        return .portrait
    }
}


class SmoochHandler: NSObject {
    private static let staticRef = SmoochHandler()

    var conversation: SKTConversation? {
        didSet {
            self.conversation?.delegate = self
        }
    }
    var navController = NavigationController()
    
    var smooch_token: String?
    var app_id: String?
    var smoochInited = false
    var properties : [String: Any]?
    var channel: FlutterMethodChannel?

    static func shared() -> SmoochHandler {
        return staticRef
    }

    func show(_ channel:FlutterMethodChannel?, progress: @escaping (Int) -> Void) {
        self.channel = channel
        UNUserNotificationCenter.current().removeAllDeliveredNotifications()
        if smoochInited {
            doShow()
        } else {
            let start = Date().timeIntervalSinceReferenceDate
            DispatchQueue.global(qos: .background).async {
                var inited = false
                while(Date().timeIntervalSinceReferenceDate - start < 30) {
                    usleep(250000)
                    if self.smoochInited {
                        DispatchQueue.main.async {
                            self.doShow()
                            progress(1)
                            inited = true
                        }
                        if inited {
                            break
                        }
                    }
                }
                if !inited {
                    progress(-1)
                }
            }
        }
    }

    private func doShow() {
        let backTitle = NSLocalizedString("navigationBackButtonTitle", tableName: "SmoochLocalizable", bundle: Bundle.main, value: "", comment: "")
        
        let vc: UIViewController? = Smooch.newConversationViewController()
        vc?.navigationItem.leftBarButtonItem = UIBarButtonItem(title: backTitle,
                                                               style: .plain,
                                                               target: self,
                                                               action: #selector(self.dismiss))

        self.navController = NavigationController(rootViewController: vc!)
        UIApplication.shared.delegate?.window??.rootViewController?.present(self.navController, animated: true, completion:  nil)

        if let properties = properties {
            SKTUser.current()?.addProperties(properties)
        }
        
        self.checkSession()
    }
    
    func conversation(_ conversation: SKTConversation, shouldShowInAppNotificationFor message: SKTMessage) -> Bool {
        return true
    }
    
    func checkSession() {
        channel?.invokeMethod("shouldSendStart", arguments: nil) {
            result in
            if let shouldSendStart = result as? Bool,
                let conversation = self.conversation,
                shouldSendStart {
                conversation.sendMessage(SKTMessage(text: "iniciar"))
            }
        }
    }

    @objc private func dismiss() {
        UIApplication.shared.delegate?.window??.rootViewController?.dismiss(animated: true, completion: nil)
    }

}

extension SmoochHandler: SKTConversationDelegate {

    func conversation(_ conversation: SKTConversation, willDisplay message: SKTMessage) -> SKTMessage? {
        if let text = message.text, text.contains("iniciar") || text.contains("session") || text.contains("token:") || text.contains("token2:") {
            return nil
        } else {
            return message
        }
    }

//    func conversation(_ conversation: SKTConversation, shouldHandle action: SKTMessageAction) -> Bool {
//        if action.type != "webview" || (action.uri?.absoluteString ?? "").contains("viaapp_terms") {
//            return true
//        }
//
//        if let uri = action.uri {
//            channel?.invokeMethod("handleMessageAction", arguments: uri)
//        }
//        return false
//    }

    func conversation(_ conversation: SKTConversation, didShow viewController: UIViewController) {
        self.checkSession()
    }

    func conversation(_ conversation: SKTConversation, unreadCountDidChange unreadCount: UInt) {
        channel?.invokeMethod("updateChatUnreadCount", arguments: unreadCount)
    }
}
