import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:smooch_plugin/smooch_status.dart';

class SmoochPlugin {
  static const MethodChannel _channel = const MethodChannel('smooch_plugin');



  static Future<SmoochStatus> initialize(
      {@required String appId,
      @required String userId,
      @required String smoochToken,
      onUpdateUnreadCount,
      onHandleMessageAction,
      onShouldSendStart}) async {
    _initializeChannel(
        onUpdateUnreadCount: onUpdateUnreadCount,
        onHandleMessageAction: onHandleMessageAction,
        onShouldSendStart: onShouldSendStart);

    final int result =
        await _channel.invokeMethod('initialize', <String, dynamic>{
      'appId': appId,
      'userId': userId,
      'smoochToken': smoochToken,
    });

    if (result == -1) {
      return SmoochStatus.not_initialized;
    } else if (result == 0) {
      return SmoochStatus.successful;
    }

    return null;
  }

  static Future<SmoochStatus> show(
      [Map<String, dynamic> userProperties]) async {
    final int result =
        await _channel.invokeMethod('show', <String, dynamic>{
      'properties': userProperties,
    });
    if (result == -1) {
      return SmoochStatus.not_initialized;
    } else if (result == 0) {
      return SmoochStatus.successful;
    }

    return null;
  }
  
  static void logout() {
    _channel.invokeMethod("logout");
  }

  static void _initializeChannel(
      {onUpdateUnreadCount, onHandleMessageAction, onShouldSendStart}) {
    _channel.setMethodCallHandler((MethodCall call) async {
      String method = call.method;
      dynamic arguments = call.arguments;
      if (method == "updateChatUnreadCount") {
        onUpdateUnreadCount(arguments);
      } else if (method == "handleMessageAction") {
        onHandleMessageAction(arguments);
      } else if (method == "shouldSendStart") {
        if (onShouldSendStart == null) {
          return false;
        }
        return onShouldSendStart();
      }
      return null;
    });
  }
}
